(module interp (lib "eopl.ss" "eopl")
  
  ;; interpreter for the PROC language, using the data structure
  ;; representation of procedures.

  ;; The \commentboxes are the latex code for inserting the rules into
  ;; the code in the book. These are too complicated to put here, see
  ;; the text, sorry. 

  (require "drscheme-init.scm")

  (require "lang.scm")
  (require "data-structures.scm")
  (require "environments.scm")

  (provide value-of-program value-of)

;;;;;;;;;;;;;;;; the interpreter ;;;;;;;;;;;;;;;;

  ;; value-of-program : Program -> ExpVal
  (define value-of-program 
    (lambda (pgm)
      (cases program pgm
        (a-program (exp1)
          (value-of exp1 (init-env))))))

  ;; value-of : Exp * Env -> ExpVal
  (define value-of
    (lambda (exp env)
      (cases expression exp

        ;\commentbox{ (value-of (const-exp \n{}) \r) = \n{}}
        (const-exp (num) (num-val num))

        ;\commentbox{ (value-of (var-exp \x{}) \r) = (apply-env \r \x{})}
        (var-exp (var) (apply-env env var))

        ;\commentbox{\diffspec}
        (diff-exp (exp1 exp2)
          (let ((val1 (value-of exp1 env))
                (val2 (value-of exp2 env)))
            (let ((num1 (expval->num val1))
                  (num2 (expval->num val2)))
              (num-val
                (- num1 num2)))))

        ;\commentbox{\zerotestspec}
        (zero?-exp (exp1)
          (let ((val1 (value-of exp1 env)))
            (let ((num1 (expval->num val1)))
              (if (zero? num1)
                (bool-val #t)
                (bool-val #f)))))
              
        ;\commentbox{\ma{\theifspec}}
        (if-exp (exp1 exp2 exp3)
          (let ((val1 (value-of exp1 env)))
            (if (expval->bool val1)
              (value-of exp2 env)
              (value-of exp3 env))))

        ;\commentbox{\ma{\theletspecsplit}}
        (let-exp (var exp1 body)       
          (let ((val1 (value-of exp1 env)))
            (value-of body
              (extend-env var val1 env))))
        
        (proc-exp (var body)
          (proc-val (procedure (value-of-mult-id var env) body env)))

        (call-exp (rator rand)
          (let ((proc (expval->proc (value-of rator env)))
                (arg (value-of-mult-exp rand env)))
            (apply-procedure proc arg)))

        )))

    ;; value-of-mult-exp : Mult-Exp * Env -> ListOf(ExpVal)
    (define value-of-mult-exp
    (lambda (mult-exp env)
      (cases mult-expression mult-exp
        (empty-mult-exp ()
           '())
        (true-mult-exp (exp m-exp)
            (cons (value-of exp env) (value-of-mult-exp m-exp env)))
        )))

  ;; value-of-mult-id : Mult-Id * Env -> ListOf(Id)
  (define value-of-mult-id
    (lambda (mult-id env)
      (cases mult-identifier mult-id
        (empty-mult-id ()
         '())
        (true-mult-id (id mult-id-list)
           (cons id (value-of-mult-id-list mult-id-list env)))
        )))

  ;; value-of-mult-id-list : Mult-Id-List * Env -> ListOf(Id)
  (define value-of-mult-id-list
    (lambda (mult-id-list env)
      (cases mult-identifier-list mult-id-list
        (empty-mult-id-list ()
           '())
        (true-mult-id-list (id m-id-list)
           (cons id (value-of-mult-id-list m-id-list env)))
        )))
  
  

  ;; extend-env-list : ListOf(Id) * ListOf(ExpVal) Env -> Env
  (define (extend-env-list lovar loval saved-env)
    (cond 
      [(null? lovar) saved-env]
      [else (extend-env (car lovar) (car loval) (extend-env-list (cdr lovar) (cdr loval) saved-env))]))
  
  ;; apply-procedure : Proc * ListOf(ExpVal) -> ExpVal
  (define apply-procedure
    (lambda (proc1 val)
      (cases proc proc1
        (procedure (var body saved-env)
          (value-of body (extend-env-list var val saved-env))))))

  )
